@@ -49,7 +49,8 @@ class Provisioner(object):
         self._api = _os_api.API(session=session, cloud_region=cloud_region)
         self._dry_run = dry_run
 
-    def reserve_node(self, resource_class, capabilities=None):
+    def reserve_node(self, resource_class=None, capabilities=None,
+                     candidates=None):
         """Find and reserve a suitable node.
 
         Example::
@@ -57,20 +58,32 @@ class Provisioner(object):
          node = provisioner.reserve_node("compute",
                                          capabilities={"boot_mode": "uefi"})
 
-        :param resource_class: Requested resource class.
+        :param resource_class: Requested resource class. If ``None``, a node
+            with any resource class can be chosen.
         :param capabilities: Requested capabilities as a dict.
+        :param candidates: List of nodes (UUIDs, names or `Node` objects)
+            to pick from. The filters (for resource class and capabilities)
+            are still applied to the provided list. The order in which
+            the nodes are considered is retained.
         :return: reserved `Node` object.
         :raises: :py:class:`metalsmith.exceptions.ReservationFailed`
         """
         capabilities = capabilities or {}
 
-        nodes = self._api.list_nodes(resource_class=resource_class)
+        if candidates:
+            nodes = [self._api.get_node(node) for node in candidates]
+            if resource_class:
+                nodes = [node for node in nodes
+                         if node.resource_class == resource_class]
+        else:
+            nodes = self._api.list_nodes(resource_class=resource_class)
+            # Ensure parallel executions don't try nodes in the same sequence
+            random.shuffle(nodes)
+
         if not nodes:
             raise exceptions.ResourceClassNotFound(resource_class,
                                                    capabilities)
 
-        # Make sure parallel executions don't try nodes in the same sequence
-        random.shuffle(nodes)
         LOG.debug('Ironic nodes: %s', nodes)
 
         filters = [_scheduler.CapabilitiesFilter(resource_class, capabilities),
@@ -80,6 +93,10 @@ class Provisioner(object):
                                              capabilities)
         node = _scheduler.schedule_node(nodes, filters, reserver,
                                         dry_run=self._dry_run)
+        if capabilities:
+            node = self._api.update_node(
+                node, {'/instance_info/capabilities': capabilities})
+
         LOG.debug('Reserved node: %s', node)
         return node
 
@@ -146,7 +163,8 @@ class Provisioner(object):
         return hostname
 
     def provision_node(self, node, image, nics=None, root_disk_size=None,
-                       config=None, hostname=None, netboot=False, wait=None):
+                       config=None, hostname=None, netboot=False,
+                       capabilities=None, wait=None):
         """Provision the node with the given image.
 
         Example::
@@ -172,6 +190,10 @@ class Provisioner(object):
         :param hostname: Hostname to assign to the instance. Defaults to the
             node's name or UUID.
         :param netboot: Whether to use networking boot for final instances.
+        :param capabilities: Requested capabilities of the node. If present,
+            overwrites the capabilities set by :meth:`reserve_node`.
+            Note that the capabilities are not checked against the ones
+            provided by the node - use :meth:`reserve_node` for that.
         :param wait: How many seconds to wait for the deployment to finish,
             None to return immediately.
         :return: :py:class:`metalsmith.Instance` object with the current
@@ -200,6 +222,9 @@ class Provisioner(object):
 
             nics = self._get_nics(nics or [])
 
+            if capabilities is None:
+                capabilities = node.instance_info.get('capabilities') or {}
+
             if self._dry_run:
                 LOG.warning('Dry run, not provisioning node %s',
                             _utils.log_node(node))
@@ -208,11 +233,11 @@ class Provisioner(object):
             self._create_and_attach_ports(node, nics,
                                           created_ports, attached_ports)
 
-            target_caps = {'boot_option': 'netboot' if netboot else 'local'}
+            capabilities['boot_option'] = 'netboot' if netboot else 'local'
 
             updates = {'/instance_info/image_source': image.id,
                        '/instance_info/root_gb': root_disk_size,
-                       '/instance_info/capabilities': target_caps,
+                       '/instance_info/capabilities': capabilities,
                        '/extra/%s' % _CREATED_PORTS: created_ports,
                        '/extra/%s' % _ATTACHED_PORTS: attached_ports,
                        '/instance_info/%s' % _os_api.HOSTNAME_FIELD: hostname}
@@ -251,13 +276,14 @@ class Provisioner(object):
             LOG.debug('Waiting for node %(node)s to reach state active '
                       'with timeout %(timeout)s',
                       {'node': _utils.log_node(node), 'timeout': wait})
-            node = self.wait_for_provisioning([node], timeout=wait)[0]
+            instance = self.wait_for_provisioning([node], timeout=wait)[0]
             LOG.info('Deploy succeeded on node %s', _utils.log_node(node))
         else:
             # Update the node to return it's latest state
             node = self._api.get_node(node, refresh=True)
+            instance = _instance.Instance(self._api, node)
 
-        return _instance.Instance(self._api, node)
+        return instance
 
     def _get_nics(self, nics):
         """Validate and get the NICs."""
@@ -358,12 +384,14 @@ class Provisioner(object):
             to finish provisioning. If ``None`` (the default), wait forever
             (more precisely, until the operation times out on server side).
         :param delay: Delay (in seconds) between two provision state checks.
-        :return: List of updated nodes if all succeeded.
+        :return: List of updated :py:class:`metalsmith.Instance` objects if
+            all succeeded.
         :raises: :py:class:`metalsmith.exceptions.DeploymentFailure`
             if the deployment failed or timed out for any nodes.
         """
-        return self._wait_for_state(nodes, 'active',
-                                    timeout=timeout, delay=delay)
+        nodes = self._wait_for_state(nodes, 'active',
+                                     timeout=timeout, delay=delay)
+        return [_instance.Instance(self._api, node) for node in nodes]
 
     def _wait_for_state(self, nodes, state, timeout, delay=15):
         if timeout is not None and timeout <= 0:
